@@ -46,8 +46,9 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
-#include <linux/unistd.h>	/* _syscall */
 #include "nls.h"
+#include "blkdev.h"
+#include "linux_version.h"
 #include "common.h"
 
 #include "gpt.h"
@@ -246,7 +247,7 @@ get_sector(char *dev, int fd, unsigned long sno) {
 
 static int
 msdos_signature (struct sector *s) {
-    unsigned char *data = s->data;
+    unsigned char *data = (unsigned char *)s->data;
     if (data[510] == 0x55 && data[511] == 0xaa)
 	    return 1;
     error(_("ERROR: sector %lu does not have an msdos signature\n"),
@@ -297,41 +298,49 @@ static int
 save_sectors(char *dev, int fdin) {
     struct sector *s;
     char ss[516];
-    int fdout;
+    int fdout = -1;
 
     fdout = open(save_sector_file, O_WRONLY | O_CREAT, 0444);
     if (fdout < 0) {
 	perror(save_sector_file);
 	error(_("cannot open partition sector save file (%s)\n"),
 	       save_sector_file);
-	return 0;
+	goto err;
     }
 
     for (s = sectorhead; s; s = s->next)
 	if (s->to_be_written) {
 	    ulong_to_chars(s->sectornumber, ss);
 	    if (!sseek(dev, fdin, s->sectornumber))
-		return 0;
+		goto err;
 	    if (read(fdin, ss+4, 512) != 512) {
 		perror("read");
 		error(_("read error on %s - cannot read sector %lu\n"),
 		       dev, s->sectornumber);
-		return 0;
+		goto err;
 	    }
 	    if (write(fdout, ss, sizeof(ss)) != sizeof(ss)) {
 		perror("write");
 		error(_("write error on %s\n"), save_sector_file);
-		return 0;
+		goto err;
 	    }
 	}
+
+    close(fdout);
     return 1;
+
+err:
+    if (fdout >= 0)
+	close(fdout);
+    return 0;
 }
 
 static void reread_disk_partition(char *dev, int fd);
 
 static int
 restore_sectors(char *dev) {
-    int fdin, fdout, ct;
+    int fdin = -1, fdout = -1;
+    int ct;
     struct stat statbuf;
     char *ss0, *ss;
     unsigned long sno;
@@ -340,54 +349,63 @@ restore_sectors(char *dev) {
 	perror(restore_sector_file);
 	error(_("cannot stat partition restore file (%s)\n"),
 	       restore_sector_file);
-	return 0;
+	goto err;
     }
     if (statbuf.st_size % 516) {
 	error(_("partition restore file has wrong size - not restoring\n"));
-	return 0;
+	goto err;
     }
     if (!(ss = (char *) malloc(statbuf.st_size))) {
 	error(_("out of memory?\n"));
-	return 0;
+	goto err;
     }
     fdin = open(restore_sector_file, O_RDONLY);
     if (fdin < 0) {
 	perror(restore_sector_file);
 	error(_("cannot open partition restore file (%s)\n"),
 	       restore_sector_file);
-	return 0;
+	goto err;
     }
     if (read(fdin, ss, statbuf.st_size) != statbuf.st_size) {
 	perror("read");
 	error(_("error reading %s\n"), restore_sector_file);
-	return 0;
+	goto err;
     }
 
     fdout = open(dev, O_WRONLY);
     if (fdout < 0) {
 	perror(dev);
 	error(_("cannot open device %s for writing\n"), dev);
-	return 0;
+	goto err;
     }
 
     ss0 = ss;
     ct = statbuf.st_size/516;
     while(ct--) {
-	sno = chars_to_ulong(ss);
+	sno = chars_to_ulong((unsigned char *) ss);
 	if (!sseek(dev, fdout, sno))
-	  return 0;
+	  goto err;
 	if (write(fdout, ss+4, 512) != 512) {
 	    perror(dev);
 	    error(_("error writing sector %lu on %s\n"), sno, dev);
-	    return 0;
+	    goto err;
 	}
 	ss += 516;
     }
     free(ss0);
 
     reread_disk_partition(dev, fdout);
+    close(fdin);
 
     return 1;
+
+err:
+    if (fdin >= 0)
+	close(fdin);
+    if (fdout >= 0)
+	close(fdout);
+
+    return 0;
 }
 
 /*
@@ -441,7 +459,7 @@ get_geometry(char *dev, int fd, int silent) {
     R.cylinders = 0;
     R.total_size = 0;
 
-    if (disksize(fd, &sectors)) {
+    if (blkdev_get_sectors(fd, &sectors) == -1) {
 	/* maybe an ordinary file */
 	struct stat s;
 
@@ -699,8 +717,8 @@ copy_to_part(char *cp, struct partition *p) {
     p->end_chs.h = *cp++;
     p->end_chs.s = *cp++;
     p->end_chs.c = *cp++;
-    p->start_sect = copy_to_int(cp);
-    p->nr_sects = copy_to_int(cp+4);
+    p->start_sect = copy_to_int((unsigned char *) cp);
+    p->nr_sects = copy_to_int((unsigned char *) cp+4);
 }
 
 static void
@@ -1482,22 +1500,6 @@ bsd_partition(char *dev, int fd, struct part_desc *ep, struct disk_desc *z) {
 	z->partno = pno;
 }
 
-#define MAKE_VERSION(p,q,r)     (65536*(p) + 256*(q) + (r))
-
-static int
-linux_version_code(void) {
-        struct utsname my_utsname;
-        int p, q, r;
-
-        if (uname(&my_utsname) == 0) {
-                p = atoi(strtok(my_utsname.release, "."));
-                q = atoi(strtok(NULL, "."));
-                r = atoi(strtok(NULL, "."));
-                return MAKE_VERSION(p,q,r);
-        }
-        return 0;
-}
-
 static int
 msdos_partition(char *dev, int fd, unsigned long start, struct disk_desc *z) {
     int i;
@@ -1506,7 +1508,7 @@ msdos_partition(char *dev, int fd, unsigned long start, struct disk_desc *z) {
     struct sector *s;
     struct part_desc *partitions = &(z->partitions[0]);
     int pno = z->partno;
-    int bsd_later = (linux_version_code() >= MAKE_VERSION(2,3,40));
+    int bsd_later = (get_linux_version() >= KERNEL_VERSION(2,3,40));
 
     if (!(s = get_sector(dev, fd, start)))
 	return 0;
@@ -1694,8 +1696,8 @@ struct dumpfld {
 #define RD_CMD (-2)
 
 static int
-read_stdin(unsigned char **fields, unsigned char *line, int fieldssize, int linesize) {
-    unsigned char *lp, *ip;
+read_stdin(char **fields, char *line, int fieldssize, int linesize) {
+    char *lp, *ip;
     int c, fno;
 
     /* boolean true and empty string at start */
@@ -1990,8 +1992,8 @@ build_surrounding_extended(struct part_desc *p, struct part_desc *ep,
 static int
 read_line(int pno, struct part_desc *ep, char *dev, int interactive,
 	  struct disk_desc *z) {
-    unsigned char line[1000];
-    unsigned char *fields[11];
+    char line[1000];
+    char *fields[11];
     int fno, pct = pno%4;
     struct part_desc p, *orig;
     unsigned long ff, ff1, ul, ml, ml1, def;
@@ -2281,7 +2283,7 @@ read_input(char *dev, int interactive, struct disk_desc *z) {
  */
 
 static void version(void) {
-    printf("sfdisk (%s)", PACKAGE_STRING);
+    printf("sfdisk (%s)\n", PACKAGE_STRING);
 }
 
 static void
@@ -2415,17 +2417,9 @@ is_ide_cdrom_or_tape(char *device) {
 }
 
 #define PROC_PARTITIONS	"/proc/partitions"
-static FILE *procf = NULL;
-
-static void
-openproc(void) {
-	procf = fopen(PROC_PARTITIONS, "r");
-	if (procf == NULL)
-		fprintf(stderr, _("cannot open %s\n"), PROC_PARTITIONS);
-}
 
 static char *
-nextproc(void) {
+nextproc(FILE *procf) {
 	static char devname[120];
 	char line[100], ptname[100];
 	int ma, mi, sz;
@@ -2442,10 +2436,8 @@ nextproc(void) {
 		return devname;
 	}
 
-	fclose(procf);
-	procf = NULL;
 	return NULL;
-}	
+}
 
 static void
 gpt_warning(char *dev, int warn_only)
@@ -2603,21 +2595,29 @@ main(int argc, char **argv) {
 
     if (optind == argc &&
 	(opt_list || opt_out_geom || opt_out_pt_geom || opt_size || verify)) {
+	FILE *procf;
+
 	/* try all known devices */
 	total_size = 0;
-	openproc();
-	while ((dev = nextproc()) != NULL) {
-	    if (is_ide_cdrom_or_tape(dev))
-		continue;
-	    gpt_warning(dev, 1);
-	    if (opt_out_geom)
-		do_geom(dev, 1);
-	    if (opt_out_pt_geom)
-		do_pt_geom(dev, 1);
-	    if (opt_size)
-		do_size(dev, 1);
-	    if (opt_list || verify)
-		do_list(dev, 1);
+
+	procf = fopen(PROC_PARTITIONS, "r");
+	if (!procf)
+	    fprintf(stderr, _("cannot open %s\n"), PROC_PARTITIONS);
+	else {
+	    while ((dev = nextproc(procf)) != NULL) {
+	        if (is_ide_cdrom_or_tape(dev))
+		   continue;
+	        gpt_warning(dev, 1);
+	        if (opt_out_geom)
+		   do_geom(dev, 1);
+	        if (opt_out_pt_geom)
+		   do_pt_geom(dev, 1);
+	        if (opt_size)
+		   do_size(dev, 1);
+	        if (opt_list || verify)
+		   do_list(dev, 1);
+	    }
+	    fclose(procf);
 	}
 
 	if (opt_size)
@@ -2732,6 +2732,8 @@ do_list (char *dev, int silent) {
 	else
 	  exit_status = 1;
     }
+
+    close(fd);
 }
 
 static void
@@ -2747,6 +2749,8 @@ do_geom (char *dev, int silent) {
     if (R.cylinders)
 	printf(_("%s: %ld cylinders, %ld heads, %ld sectors/track\n"),
 	       dev, R.cylinders, R.heads, R.sectors);
+
+    close(fd);
 }
 
 static void
@@ -2778,6 +2782,8 @@ do_pt_geom (char *dev, int silent) {
     if (R.cylinders)
 	printf(_("%s: %ld cylinders, %ld heads, %ld sectors/track\n"),
 	       dev, R.cylinders, R.heads, R.sectors);
+
+    close(fd);
 }
 
 /* for compatibility with earlier fdisk: provide option -s */
@@ -2790,7 +2796,7 @@ do_size (char *dev, int silent) {
     if (fd < 0)
 	return;
 
-    if (disksize(fd, &size)) {
+    if (blkdev_get_sectors(fd, &size) == -1) {
 	if (!silent) {
 	    perror(dev);
 	    fatal(_("Cannot get size of %s\n"), dev);
@@ -2810,6 +2816,8 @@ do_size (char *dev, int silent) {
       printf("%llu\n", size);
 
     total_size += size;
+
+    close(fd);
 }
 
 /*
@@ -2900,6 +2908,8 @@ do_activate (char **av, int ac, char *arg) {
     if (i != 1)
       warn(_("You have %d active primary partitions. This does not matter for LILO,\n"
 	   "but the DOS MBR will only boot a disk with 1 active partition.\n"), i);
+
+    close(fd);
 }
 
 static void
@@ -2945,6 +2955,8 @@ do_unhide (char **av, int ac, char *arg) {
       warn(_("Done\n\n"));
     else
       exit_status = 1;
+
+    close(fd);
 }
 
 static void
@@ -2976,6 +2988,8 @@ do_change_id(char *dev, char *pnam, char *id) {
       warn(_("Done\n\n"));
     else
       exit_status = 1;
+
+    close(fd);
 }
 
 static void
@@ -2985,6 +2999,8 @@ do_reread(char *dev) {
     fd = my_open(dev, 0, 0);
     if (reread_ioctl(fd))
       do_warn(_("This disk is currently in use.\n"));
+
+    close(fd);
 }
 
 /*
@@ -3082,11 +3098,11 @@ do_fdisk(char *dev){
       exit_status = 1;
 
     reread_disk_partition(dev, fd);
-    
+
     warn(_("If you created or changed a DOS partition, /dev/foo7, say, then use dd(1)\n"
 	 "to zero the first 512 bytes:  dd if=/dev/zero of=/dev/foo7 bs=512 count=1\n"
 	 "(See fdisk(8).)\n"));
-    
+
     sync();			/* superstition */
     sleep(3);
     exit(exit_status);
